package com.ysnows.base.widget.tablayout;

import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.view.View;

import com.chad.library.adapter.base.BaseQuickAdapter;
import com.chad.library.adapter.base.listener.OnItemClickListener;
import com.qmuiteam.qmui.widget.tab.QMUITab;
import com.ysnows.base.R;

import java.util.ArrayList;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

/**
 * Created by xianguangjin on 16/5/20.
 */
public class YTabLayout extends RecyclerView implements OnItemClickListener {
    private ArrayList<YTab> tabs = new ArrayList<>();

    private int curTab = 0;
    private int preTab = 0;
    private MenuTabBuilder mTabBuilder;
    private MainMenuAdapter adapter;
    private GridLayoutManager layoutManager;

    public YTabLayout(Context context) {
        super(context);
        init(context, null, 0);
    }

    public YTabLayout(Context context, AttributeSet attrs) {
        super(context, attrs);
        init(context, attrs, 0);
    }

    public YTabLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init(context, attrs, defStyleAttr);
    }

    private void init(Context context, AttributeSet attrs, int defStyleAttr) {
        setHasFixedSize(true);
        TypedArray array = context.obtainStyledAttributes(attrs,
                R.styleable.YTabLayout, defStyleAttr, 0);

        // tabBuilder
        int normalTextSize = array.getDimensionPixelSize(
                R.styleable.YTabLayout_android_textSize,
                getResources().getDimensionPixelSize(R.dimen.qmui_tab_segment_text_size));

        normalTextSize = array.getDimensionPixelSize(
                R.styleable.YTabLayout_normal_text_size, normalTextSize);
        int selectedTextSize = normalTextSize;
        selectedTextSize = array.getDimensionPixelSize(
                R.styleable.YTabLayout_selected_text_size, selectedTextSize);

        mTabBuilder = new MenuTabBuilder(context)
                .setTextSize(normalTextSize, selectedTextSize)
                .setIconPosition(array.getInt(R.styleable.YTabLayout_icon_position,
                        QMUITab.ICON_POSITION_LEFT));

        array.recycle();

        adapter = new MainMenuAdapter();
        setAdapter(adapter);
        adapter.setOnItemClickListener(this);
        adapter.setAnimationEnable(true);
        adapter.setAnimationFirstOnly(false);
        adapter.setAnimationWithDefault(BaseQuickAdapter.AnimationType.ScaleIn);
    }


    public void rendor() {
        layoutManager = new GridLayoutManager(getContext(), tabs.size());
        //setLayoutManager(new LinearLayoutManager(getContext(), HORIZONTAL, false));
        setLayoutManager(layoutManager);
        adapter.setList(tabs);
    }

    public YTabLayout addTab(YTab tab) {
        tabs.add(tab);
        return this;
    }


    public MenuTabBuilder tabBuilder() {
        // do not change mTabBuilder to keep common config not changed
        return new MenuTabBuilder(mTabBuilder);
    }


    public void switchTab(int tabIndex) {
        preTab = curTab;
        curTab = tabIndex;

        //超出index,不做处理
        if (tabIndex >= tabs.size()) {
            return;
        }

        FragmentManager manager = ((FragmentActivity) getContext()).getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();

        Fragment fragment;
        for (int i = 0; i < tabs.size(); i++) {
            YTab yTab = tabs.get(i);
            fragment = manager.findFragmentByTag("tab_" + i);

            if (tabIndex == i) {
                yTab.setSelected(true);
                if (fragment == null) {
                    fragment = yTab.fragment;
                    transaction.add(R.id.container, fragment, "tab_" + tabIndex);
                } else {
                    transaction.show(fragment);
                }
            } else {
                yTab.setSelected(false);
                if (fragment == null) {
                    continue;
                }
                transaction.hide(fragment);
            }
        }

        transaction.addToBackStack(getClass().getCanonicalName());
        transaction.commitAllowingStateLoss();
    }

    @Override
    public void onItemClick(@NonNull BaseQuickAdapter<?, ?> adapter, @NonNull View view, int position) {
        switchTab(position);
    }
}
